class TokenAttributeBuilderFunctionsQueryBuilder extends GeneralFunctionsQueryBuilder {
  constructor(elements) {
    super(elements);
    this.elements = elements;

    this.elements.tokenSubmitButton.addEventListener('click', () => {this.addTokenToQuery();});
    this.elements.wordInput.addEventListener('input', () => {this.optionToggleHandler();});
    this.elements.lemmaInput.addEventListener('input', () => {this.optionToggleHandler();});
  }

  lemmaOrWordInputCheck() {
    let input;

    if (document.querySelector('[data-toggle-area="word"]').classList.contains('hide') === false) {
      input = this.elements.wordInput;
    } else {
      input = this.elements.lemmaInput;
    }

    return input;
  }

  optionToggleHandler() {
    let input = this.lemmaOrWordInputCheck()

    if (input.value === '') {
      this.toggleClass(['incidence-modifiers', 'or', 'and'], 'disabled', 'add');
    } else {
      this.toggleClass(['incidence-modifiers', 'or', 'and'], 'disabled', 'remove');
    }
  }

  disableTokenSubmit() {
    this.elements.isTokenQueryInvalid = true;
    this.elements.tokenSubmitButton.classList.add('red');
    this.elements.noValueMessage.classList.remove('hide');
    setTimeout(() => {
      this.elements.tokenSubmitButton.classList.remove('red');
    }, 500);
    setTimeout(() => {
      this.elements.noValueMessage.classList.add('hide');
    }, 3000);
  }

  tokenChipFactory(prettyQueryText, tokenText) {
    tokenText = encodeURI(tokenText);
    let builderElement;
    let queryChipElement;
    builderElement = document.createElement('div');
    builderElement.innerHTML = `
    <div class='chip col s2 l2' style='margin-top:20px;' data-tokentext='${tokenText}'>
      ${prettyQueryText}
      <i class='material-icons close'>close</i>
    </div>`;
    queryChipElement = builderElement.firstElementChild;
    this.elements.tokenQuery.appendChild(queryChipElement);
  }

  addTokenToQuery() {
    let c = this.elements.ignoreCaseCheckbox.checked ? ' %c' : '';
    let tokenQueryContent = '';
    let tokenQueryText = '';
    this.elements.isTokenQueryInvalid = false;
    let tokenIsEmpty = false;

    this.elements.tokenQuery.childNodes.forEach(element => {
      tokenQueryContent += ' ' + element.firstChild.data + ' ';
      tokenQueryText += decodeURI(element.dataset.tokentext);
      if (element.innerText.indexOf('empty token') !== -1) {
        tokenIsEmpty = true;
      }
    });

    switch (this.elements.positionalAttrSelection.value) {
      case 'word':
        if (this.elements.wordInput.value === '') {
          this.disableTokenSubmit();
        } else {
          tokenQueryContent += `word=${this.elements.wordInput.value}${c}`;
          tokenQueryText += `word="${this.elements.wordInput.value}"${c}`;
          this.elements.wordInput.value = '';
        }
        break;
      case 'lemma':
        if (this.elements.lemmaInput.value === '') {
          this.disableTokenSubmit();
        } else {
          tokenQueryContent += `lemma=${this.elements.lemmaInput.value}${c}`;
          tokenQueryText += `lemma="${this.elements.lemmaInput.value}"${c}`;
          this.elements.lemmaInput.value = '';
        }
        break;
      case 'english-pos':
        if (this.elements.englishPos.value === 'default') {
          this.disableTokenSubmit();
        } else {
          tokenQueryContent += `pos=${this.elements.englishPosSelection.value}`;
          tokenQueryText += `pos="${this.elements.englishPosSelection.value}"`;
          this.elements.englishPosSelection.value = '';
        }
        break;
      case 'german-pos':
        if (this.elements.germanPosSelection.value === 'default') {
          this.disableTokenSubmit();
        } else {
          tokenQueryContent += `pos=${this.elements.germanPosSelection.value}`;
          tokenQueryText += `pos="${this.elements.germanPosSelection.value}"`;
          this.elements.germanPosSelection.value = '';
        }
        break;
      case 'simple-pos':
        if (this.elements.simplePosSelection.value === 'default') {
          this.disableTokenSubmit();
        } else {
          tokenQueryContent += `simple_pos=${this.elements.simplePosSelection.value}`;
          tokenQueryText += `simple_pos="${this.elements.simplePosSelection.value}"`;
          this.elements.simplePosSelection.value = '';
        }
        break;
      default:
        break;
    }

    // isTokenQueryInvalid looks if a valid value is passed. If the input fields/dropdowns are empty (isTokenQueryInvalid === true), no token is added.
    if (this.elements.isTokenQueryInvalid === false) {
      // Square brackets are added only if it is not an empty token (where they are already present).
      if (tokenIsEmpty === false) {
        tokenQueryText = '[' + tokenQueryText + ']';
        }
      this.queryChipFactory('token', tokenQueryContent, tokenQueryText);
      this.resetPositionalAttrModal();
      this.elements.positionalAttrModal.close();

    }
  }

  resetPositionalAttrModal() {
    let originalSelectionList = 
      `
        <option value="word" selected>word</option>
        <option value="lemma" >lemma</option>
        <option value="english-pos">english pos</option>
        <option value="german-pos">german pos</option>
        <option value="simple-pos">simple_pos</option>
        <option value="empty-token">empty token</option>
      `;
    document.querySelector('#positional-attr-selection').innerHTML = originalSelectionList;
    this.elements.tokenQuery.innerHTML = '';
    this.toggleClass(['word', 'lemma', 'english-pos', 'german-pos', 'simple-pos'], 'hide', 'add');
    this.toggleClass(['word'], 'hide', 'remove');
    this.toggleClass(['incidence-modifiers', 'or', 'and'], 'disabled', 'add');

    document.querySelector(`#positional-attr-selection option[value="word"]`).selected = true;

    let instance = M.FormSelect.getInstance(document.getElementById('positional-attr-selection'));
    instance.destroy();
    M.FormSelect.init(document.getElementById('positional-attr-selection'));
  }

  actionButtonHandler(elem) {
    let input = this.lemmaOrWordInputCheck();
    switch (elem) {
      case 'option-group':
        input.value  += '(option1|option2)';
        let firstIndex = input.value.indexOf('option1');
        let lastIndex = firstIndex + 'option1'.length;
        input.focus();
        input.setSelectionRange(firstIndex, lastIndex);
        break;
      case 'wildcard-char':
        input.value  += '.';
        break;
      case 'and':
        this.conditionHandler('and', " & ");
        break;
      case 'or':
        this.conditionHandler('or', " | ");
        break;
      default:
        break;
    }
    this.optionToggleHandler();
  }

  incidenceModifierHandler(elem) {
    // For word and lemma, the incidence modifiers are inserted in the input field. For the others, one or two chips are created which contain the respective value of the token and the incidence modifier.
    switch (this.elements.positionalAttrSelection.value) {
      case 'empty-token':
        this.tokenChipFactory(elem.innerText, elem.dataset.token);
        break;
      case 'english-pos':
        this.tokenChipFactory(`pos=${this.elements.englishPos.value}`, `pos="${this.elements.englishPos.value}"`);
        this.tokenChipFactory(elem.innerText, elem.dataset.token);
        this.elements.tokenQueryFilled = true;
        break;
      case 'german-pos':
        this.tokenChipFactory(`pos=${this.elements.germanPos.value}`, `pos="${this.elements.germanPos.value}"`);
        this.tokenChipFactory(elem.innerText, elem.dataset.token);
        this.elements.tokenQueryFilled = true;
        break;
      case 'simple-pos':
        this.tokenChipFactory(`simple_pos=${this.elements.simplePos.value}`, `simple_pos="${this.elements.simplePos.value}"`);
        this.tokenChipFactory(elem.innerText, elem.dataset.token);
        this.elements.tokenQueryFilled = true;
        break;     
      default:
        let input = this.lemmaOrWordInputCheck();
        input.value += elem.dataset.token;
        break;
    }
  }

  nmSubmitHandler(modalId) {
    let modal = document.querySelector(`#${modalId}`);
    let input_n = modal.querySelector('.n-m-input[data-value-type="n"]').value;
    let input_m = modalId === 'between-nm-modal' ? ',' + modal.querySelector('.n-m-input[data-value-type="m"]').value : '';
    let input = `${input_n}${input_m}`;

    let instance = M.Modal.getInstance(modal);
    instance.close();
    
    switch (this.elements.positionalAttrSelection.value) {
      case 'word':
        this.elements.wordInput.value += '{' + input + '}';
        break;
      case 'lemma':
        this.elements.lemmaInput.value += '{' + input + '}';
        break;
      default:
        break;
    }
  }

  conditionHandler(conditionText, conditionQueryContent) {
    let tokenQueryContent;
    let tokenQueryText;
    let c = this.elements.ignoreCaseCheckbox.checked ? ' %c' : '';
    let selectionDefault = "word";
    let optionDeleteList = [];

    switch (this.elements.positionalAttrSelection.value) {
      case 'word':
        tokenQueryContent = `word=${this.elements.wordInput.value}${c}`;
        tokenQueryText = `word="${this.elements.wordInput.value}"${c}`;
        this.elements.wordInput.value = '';
        if (conditionText === 'and') {
          selectionDefault = "english-pos";
          optionDeleteList = ['word', 'lemma', 'empty-token'];
        }
        break;
      case 'lemma':
        tokenQueryContent = `lemma=${this.elements.lemmaInput.value}${c}`;
        tokenQueryText = `lemma="${this.elements.lemmaInput.value}"${c}`;
        this.elements.lemmaInput.value = '';
        if (conditionText === 'and') {
          selectionDefault = "english-pos";
          optionDeleteList = ['word', 'lemma', 'empty-token'];
        }
        break;
      case 'english-pos':
        tokenQueryContent = `pos=${this.elements.englishPos.value}`;
        tokenQueryText = `pos="${this.elements.englishPos.value}"`;
        this.elements.englishPos.value = '';
        break;
      case 'german-pos':
        tokenQueryContent = `pos=${this.elements.germanPos.value}`;
        tokenQueryText = `pos="${this.elements.germanPos.value}"`;
        this.elements.germanPos.value = '';
        break;
      case 'simple-pos':
        tokenQueryContent = `simple_pos=${this.elements.simplePos.value}`;
        tokenQueryText = `simple_pos="${this.elements.simplePos.value}"`;
        this.elements.simplePos.value = '';
        break;
      default:
        break;
    }

    this.tokenChipFactory(tokenQueryContent, tokenQueryText);
    this.tokenChipFactory(conditionText, conditionQueryContent);
    this.setTokenSelection(selectionDefault, optionDeleteList);
  }

  setTokenSelection(selection, optionDeleteList) {
    optionDeleteList.forEach(option => {
      document.querySelector(`#positional-attr-selection option[value=${option}]`).remove();
    });

    document.querySelector(`#positional-attr-selection option[value=${selection}]`).selected = true;

    let instance = M.FormSelect.getInstance(document.getElementById('positional-attr-selection'));
    instance.destroy();
    M.FormSelect.init(document.getElementById('positional-attr-selection'));

    this.toggleClass(['word', 'lemma', 'english-pos', 'german-pos', 'simple-pos'], 'hide', 'add');
    this.toggleClass([selection], 'hide', 'remove');
    this.toggleClass(['incidence-modifiers', 'or', 'and'], 'disabled', 'add');
  }

}
